VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "CConsole"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False

Option Explicit

Private Declare Function AllocConsole Lib "kernel32" () As Long
Private Declare Function FreeConsole Lib "kernel32" () As Long
Private Declare Function GetStdHandle Lib "kernel32" (ByVal nStdHandle As Long) As Long
Private Declare Function CloseHandle Lib "kernel32" (ByVal hObject As Long) As Long
Private Declare Function WriteConsole Lib "kernel32" Alias "WriteConsoleA" (ByVal hConsoleOutput As Long, lpBuffer As Any, ByVal nNumberOfCharsToWrite As Long, lpNumberOfCharsWritten As Long, lpReserved As Any) As Long
Private Declare Function ReadConsole Lib "kernel32" Alias "ReadConsoleA" (ByVal hConsoleInput As Long, ByVal lpBuffer As String, ByVal nNumberOfCharsToRead As Long, lpNumberOfCharsRead As Long, lpReserved As Any) As Long
Private Declare Function SetConsoleTitle Lib "kernel32" Alias "SetConsoleTitleA" (ByVal lpConsoleTitle As String) As Long
Private Declare Function SetConsoleTextAttribute Lib "kernel32" (ByVal hConsoleOutput As Long, ByVal wAttributes As Long) As Long
Private Declare Function ReadConsoleOutputCharacter Lib "kernel32" Alias "ReadConsoleOutputCharacterA" (ByVal hConsoleOutput As Long, ByVal lpCharacter As String, ByVal nLength As Long, ByVal dwReadCoord As Long, lpNumberOfCharsRead As Long) As Long
Private Declare Function WriteConsoleOutputCharacter Lib "kernel32" Alias "WriteConsoleOutputCharacterA" (ByVal hConsoleOutput As Long, ByVal lpCharacter As String, ByVal nLength As Long, ByVal dwWriteCoord As Long, lpNumberOfCharsWritten As Long) As Long

' used for setting brightness of colors
Const BACKGROUND_INTENSITY = &H80&
Const FOREGROUND_INTENSITY = &H8&

' enum used for ForeColor property
Public Enum vbConsoleForeColors
    FOREGROUND_BLACK = &H0
    FOREGROUND_BLUE = &H1
    FOREGROUND_GREEN = &H2
    FOREGROUND_RED = &H4
    FOREGROUND_YELLOW = FOREGROUND_RED Or FOREGROUND_GREEN
    FOREGROUND_PURPLE = FOREGROUND_RED Or FOREGROUND_BLUE
    FOREGROUND_CYAN = FOREGROUND_BLUE Or FOREGROUND_GREEN
    FOREGROUND_WHITE = FOREGROUND_BLUE Or FOREGROUND_GREEN Or FOREGROUND_RED
End Enum

' enum used for BackColor property
Enum vbConsoleBackColors
    BACKGROUND_BLACK = &H0
    BACKGROUND_BLUE = &H10
    BACKGROUND_GREEN = &H20
    BACKGROUND_RED = &H40
    BACKGROUND_YELLOW = BACKGROUND_RED Or BACKGROUND_GREEN
    BACKGROUND_PURPLE = BACKGROUND_RED Or BACKGROUND_BLUE
    BACKGROUND_CYAN = BACKGROUND_BLUE Or BACKGROUND_GREEN
    BACKGROUND_WHITE = BACKGROUND_BLUE Or BACKGROUND_GREEN Or BACKGROUND_RED
End Enum

' used for attaching class to console
Enum vbStdHandles
    STD_OUTPUT_HANDLE = -11&
    STD_INPUT_HANDLE = -10&
    STD_ERROR_HANDLE = -12&
    INVALID_HANDLE_VALUE = -1&
End Enum

'#---------------------------------------------------------------------------#
'#  variables
'#---------------------------------------------------------------------------#
Private m_StdIn As Long
Private m_StdOut As Long
Private m_ForeColor As vbConsoleForeColors
Private m_BackColor As vbConsoleBackColors
Private m_BackIntense As Boolean
Private m_ForeIntense As Boolean
Private m_BufferLength As Long
Private m_Title As String

'#---------------------------------------------------------------------------#
'#  properties
'#---------------------------------------------------------------------------#
Public Property Get title() As String
    title = m_Title
End Property

Public Property Let title(ByVal New_Title As String)
    m_Title = New_Title
    ' set the new title
    SetConsoleTitle New_Title
End Property

Public Property Get ForeColor() As vbConsoleForeColors
    ForeColor = m_ForeColor
End Property

Public Property Let ForeColor(ByVal New_Color As vbConsoleForeColors)
    m_ForeColor = New_Color
    ' set the new ForeColor
    SetColors
End Property

Public Property Get BackColor() As vbConsoleBackColors
    BackColor = m_BackColor
End Property

Public Property Let BackColor(ByVal New_Color As vbConsoleBackColors)
    m_BackColor = New_Color
    ' set the new BackColor
    SetColors
End Property

Public Property Get BackIntense() As Boolean
    BackIntense = m_BackIntense
End Property

Public Property Let BackIntense(ByVal New_Value As Boolean)
    m_BackIntense = New_Value
    'set the new intensity
    SetColors
End Property

Public Property Get ForeIntense() As Boolean
    ForeIntense = m_ForeIntense
End Property

Public Property Let ForeIntense(ByVal New_Value As Boolean)
    m_ForeIntense = New_Value
    'set the new intensity
    SetColors
End Property

Public Property Get BufferLength() As String
    BufferLength = m_BufferLength
End Property

Public Property Let BufferLength(ByVal New_BufferLength As String)
    m_BufferLength = New_BufferLength
End Property

'#---------------------------------------------------------------------------#
'#  methods
'#---------------------------------------------------------------------------#
Public Sub WriteText(ByVal sText As String)
Dim lTmp As Long
    ' write the text out the the console
    If m_StdOut <> INVALID_HANDLE_VALUE Then
        WriteConsole m_StdOut, ByVal sText, Len(sText), lTmp, ByVal 0&
    Else
        Err.Raise 10103, "WriteText", "STD_OUT_HANDLE was invalid"
    End If
End Sub

Public Function ConsoleRead() As String
Dim sUserInput As String * 256
Call ReadConsole(m_StdIn, sUserInput, Len(sUserInput), vbNull, vbNull)
'Trim off the NULL charactors and the CRLF.
ConsoleRead = Left$(sUserInput, InStr(sUserInput, Chr$(0)) - 3)
End Function

Public Function WordScan() As String
    If m_StdOut <> INVALID_HANDLE_VALUE Then
        
        Dim sc_buffer As String * 9
        Dim sc_numberread As Long
        Dim numbertoread As Long
        
        Dim startcoord As Long
        startcoord = 0
        
        Call ReadConsoleOutputCharacter(m_StdOut, sc_buffer, 9, startcoord, sc_numberread)
        WordScan = sc_buffer
        
    Else
        MsgBox ("There is an unexpected error")
    End If

End Function

Public Function InsertWord(ByVal sText As String)
    
    If m_StdOut <> INVALID_HANDLE_VALUE Then
        
        Dim sc_numberread As Long
        Dim startcoord As Long
        
        WriteConsoleOutputCharacter m_StdOut, ByVal sText, Len(sText), startcoord, sc_numberread
    
    Else
        MsgBox ("There is an unexpected error")
    End If

End Function

Public Sub StartApp(ByVal sFilepath As String)
On Error Resume Next
    Shell sFilepath
End Sub

'#---------------------------------------------------------------------------#
'#  private internals
'#---------------------------------------------------------------------------#

'//--[SetColors]------------------------------------------------------------//
'
'  Calls the SetConsoleTextAttributes API to alter the fore
'  and back colors of the new console window
'
Private Sub SetColors()
Dim lTmp As Long
    If m_StdOut <> INVALID_HANDLE_VALUE Then
        ' set the intensity stuff
        If m_BackIntense Then
            lTmp = BACKGROUND_INTENSITY
        End If
        If m_ForeIntense Then
            lTmp = lTmp Or FOREGROUND_INTENSITY
        End If
        ' set all four color attributes
        SetConsoleTextAttribute m_StdOut, m_BackColor Or m_ForeColor Or lTmp
    Else
        Err.Raise 10103, "SetColors", "STD_OUT_HANDLE was invalid"
    End If
End Sub

'#---------------------------------------------------------------------------#
'#  class events
'#---------------------------------------------------------------------------#
Private Sub Class_Initialize()
    ' create the console
    If AllocConsole Then
        ' get the handles
        m_StdIn = GetStdHandle(STD_INPUT_HANDLE)
        m_StdOut = GetStdHandle(STD_OUTPUT_HANDLE)
        If m_StdIn = INVALID_HANDLE_VALUE Or _
            m_StdOut = INVALID_HANDLE_VALUE Then
            ' whoops - something went wrong somewhere
            Err.Raise 10101, "Class_Initialize", "Couldn't get STD handles"
        End If
    Else
        ' failed to allocate a console
        Err.Raise 10102, "Class_Initialize", "Couldn't allocate console"
    End If
End Sub

Private Sub Class_Terminate()
    ' close the handles
    CloseHandle m_StdIn
    CloseHandle m_StdOut
    ' destroy the console
    FreeConsole
End Sub


